@@ -75,6 +75,10 @@
 	+	replaced sleep(x) with select(1,0,0,0,timeout)
 	+	added getifaddrs() for recent BSD systems (Peter Stromberg)
 	+	added new SIOCGIFCONF for older BSD systems (Peter Stromberg)
+	
+	+	November 2003 (johna)
+	+	added <CR> into all http requests along with <LF>
+	+	added SIGCHLD handler to reap zombies
 */			
 /////////////////////////////////////////////////////////////////////////////                                            
 
@@ -97,6 +101,7 @@
 #include <syslog.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <netdb.h>
 #include <arpa/inet.h>
 #include <net/if.h>
@@ -139,7 +144,7 @@
 #define LINELEN 	        256
 #define BIGBUFLEN		16384
 
-#define VERSION			"2.0.12"
+#define VERSION			"2.0.15"
 #define NOIP_NAME		"dynupdate.no-ip.com"
 #define USER_AGENT		"User-Agent: Linux DUC "VERSION
 #define SETTING_SCRIPT		"settings.php?"
@@ -392,6 +397,7 @@ unsigned char EncodeTable[] =
 ///////////////////////////////////////////////////////////////////////////
 void	process_options(int argc, char *argv[]);
 void	alarm_handler(int signum);
+void	sigchld_handler(int signum);
 void	exit_handler(int signum);
 volatile void	wake_handler(int signum);
 void	save_IP();
@@ -520,6 +526,8 @@ int main(int argc, char *argv[])
 	    sigaction(SIGINT,&sig,NULL);
 	    sigaction(SIGTERM,&sig,NULL);
 	    sigaction(SIGQUIT,&sig,NULL);
+	    sig.sa_handler = (void *)sigchld_handler;
+	    sigaction(SIGCHLD, &sig, NULL);
 	    if (run_as_background() == FATALERR) {	// get shmem failure
 		return FATALERR;
 	    }
@@ -654,6 +662,13 @@ void alarm_handler(int signum)	// entered on SIGALRM
 	timed_out = 1;
 }
 ///////////////////////////////////////////////////////////////////////////
+void sigchld_handler(int signum)	//entered on death of child (SIGCHLD)
+{	
+	int	status;
+
+	waitpid(-1, &status, WNOHANG | WUNTRACED);
+}
+///////////////////////////////////////////////////////////////////////////
 void exit_handler(int signum)	//entered on SIGINT,  SIGTERM & SIGQUIT
 {	
 	background = 0;
@@ -666,14 +681,21 @@ volatile void wake_handler(int signum)	//entered on SIGUSR1
 ///////////////////////////////////////////////////////////////////////////
 void save_IP()
 {
-	lseek(config_fd, 0, SEEK_SET);	// re-position at start of file
-	if (write(config_fd, my_instance->Last_IP_Addr, 
-				strlen(my_instance->Last_IP_Addr) + 1) < 0) 
-	close(config_fd);
+	// config_file is opened as root, read-only permission has no effect
+	if (access(config_filename, W_OK) ==0) {
+	    lseek(config_fd, 0, SEEK_SET);	// re-position at start of file
+	    write(config_fd, my_instance->Last_IP_Addr, 
+				strlen(my_instance->Last_IP_Addr) + 1); 
 #ifdef DEBUG
-	if (my_instance->debug) 
-            Msg("Saving Last_IP_Addr %s", my_instance->Last_IP_Addr);
+	    if (my_instance->debug) 
+        	Msg("Saving Last_IP_Addr %s", my_instance->Last_IP_Addr);
+	} else {
+	    if (my_instance->debug) 
+        	Msg("Read-only file '%s' prevents saving Last_IP_Addr %s", 
+				config_filename, my_instance->Last_IP_Addr);
 #endif
+	}
+	close(config_fd);
 	my_instance->pid = 0;		// untag instance
 	shmdt(shmaddr);			// done with our shared memory
 	exit(0);
@@ -1300,7 +1322,7 @@ void get_our_visible_IPaddr(char *dest)
 	} else {
 	    // set up text for web page query
 	    sprintf(buffer, 
-		"GET http://%s/ip.php HTTP/1.0\n%s\n\n", NOIP_NAME,USER_AGENT);
+		"GET http://%s/ip.php HTTP/1.0\r\n%s\r\n\r\n", NOIP_NAME,USER_AGENT);
 	    if ((x = converse_with_web_server()) < 0) {
 		handle_dynup_error(x);
 	    } else {
@@ -1348,7 +1370,7 @@ int dynamic_update()
 	pos = &buffer[BIGBUFLEN / 2];
 	bencode(buffer, pos);
 	sprintf(buffer, 
-	    "GET http://%s/%s?%s%s HTTP/1.0\n%s\n\n",
+	    "GET http://%s/%s?%s%s HTTP/1.0\r\n%s\r\n\r\n",
 		NOIP_NAME, UPDATE_SCRIPT, REQUEST, pos, USER_AGENT);
 	x = converse_with_web_server();
 	if (x < 0)	{
@@ -1883,6 +1905,8 @@ int get_all_device_names(unsigned char *devs)
 	dq = devs;     // point at name list
 	for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
 		if (ifa->ifa_addr->sa_family == AF_LINK) {
+// FreeBSD doesn't define some of these!
+#ifdef IFT_PFLOG
 			struct if_data *ifd = (struct if_data *) ifa->ifa_data;
 			if (ifd->ifi_type == IFT_PFLOG
 				 || ifd->ifi_type == IFT_PFSYNC
@@ -1891,6 +1915,7 @@ int get_all_device_names(unsigned char *devs)
 				 || ifd->ifi_type == IFT_OTHER
 				 || ifd->ifi_type == IFT_GIF)
 		 	   continue;
+#endif
 			q = dq;     // add new name into list
 			p = ifa->ifa_name;
 			devnum++;
@@ -2065,7 +2090,7 @@ void autoconf()
 	    Msg(CMSG27);
 	    return;
 	}
-        sprintf(buffer, "GET http://%s/%s\n\n", NOIP_NAME, GETIP_SCRIPT);
+        sprintf(buffer, "GET http://%s/%s\r\n\r\n", NOIP_NAME, GETIP_SCRIPT);
         if ((x = converse_with_web_server()) <= 0) {
             handle_dynup_error(x);
 	    return;
@@ -2092,7 +2117,7 @@ void autoconf()
 	}
 	sprintf(line, "%s%s%s%s", USTRNG, login, PWDSTRNG, password);
 	bencode(line, encline);
-        sprintf(buffer, "GET http://%s/%s%s%s\n\n", 
+        sprintf(buffer, "GET http://%s/%s%s%s\r\n\r\n", 
                 		NOIP_NAME, SETTING_SCRIPT, REQUEST, encline);
         if ((x = converse_with_web_server()) <= 0) {
             handle_dynup_error(x);
